using System.ComponentModel;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Grid.BasketViewer.BusinessLevel
{
    internal class InstrumentClass : INotifyPropertyChanged
    {
        private readonly Basket _basket;
        private readonly ThreadSafeBindingList<Industry> _industries = new ThreadSafeBindingList<Industry>();
        private readonly string _instrumentClassName;
        private decimal? _marketValue;

        public InstrumentClass(string instrumentClassName, Basket basket)
        {
            _instrumentClassName = instrumentClassName;
            _basket = basket;
        }

        public string InstrumentClassName
        {
            get { return _instrumentClassName; }
        }

        //Use 2 decimals for shares 
        [Format("### ### ###.00", null, " ")]
        public decimal MarketValueEur
        {
            get { return MarketValue; }
        }

        [Format("### ### ###.00")]
        public decimal MarketValue
        {
            get
            {
                //Cache purposes: compute industry type's market value
                if (_marketValue == null)
                {
                    _marketValue = 0;
                    foreach (Industry industry in _industries)
                    {
                        _marketValue += industry.MarketValue;
                    }
                }
                return _marketValue.Value;
            }
        }

        [Format("#.00", null, " %")]
        public decimal MarketPartPercent
        {
            get { return 100*MarketPart; }
        }

        public decimal MarketPart
        {
            get
            {
                if (_basket.BasketValue != 0)
                {
                    return MarketValue/_basket.BasketValue;
                }
                return 0;
            }
        }

        [HierarchicalField]
        public ThreadSafeBindingList<Industry> Industries
        {
            get { return _industries; }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Private methods

        private void OnIndustryPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "MarketValue":
                    _marketValue = null;
                    FirePropertyChanged("MarketValue");
                    FirePropertyChanged("MarketValueEur");
                    break;
            }
        }

        #endregion

        public void AddIndustry(Industry industryType)
        {
            if (industryType != null)
            {
                _industries.Add(industryType);
                industryType.PropertyChanged += OnIndustryPropertyChanged;
                _marketValue = null;
            }
        }

        private void FirePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}